#pragma once
#include <J3ML/LinearAlgebra/Vector2.hpp>
#include <vector>
#include <chrono>

namespace Engine {
    class Entity;
}

class Engine::Entity {
protected:
    // nullptr means no parent.
    Entity* parent = nullptr;

    // Epoch micro-seconds.
    long creation_time = 0.0f;
    std::vector<Entity*> children{};
    void UpdateChildren();
public:
    // Parent child structure.
    [[nodiscard]] bool AppendChild(Entity* entity);
    void DestroyChild(Entity* entity);
    void RemoveChild(Entity* entity);
public:
    [[nodiscard]] std::vector<Entity*> GetChildren() const;
    [[nodiscard]] Entity* GetParent() const;

    // Microseconds.
    [[nodiscard]] long GetCreationTime() const;
    [[nodiscard]] long GetAge() const;
public:
    virtual void Update() {}
public:
    virtual ~Entity();
    explicit Entity() : creation_time(std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now().time_since_epoch()).count()) {}
};