#pragma once

#include <vector>
#include <Engine/types/InstancedTexture.h>
#include <Engine/types/entity/Hud.h>
#include <Engine/types/scene/Fixed.h>
#include <JGL/types/RenderTarget.h>

namespace Engine {
    class Camera;
    class Scene;
}

using namespace JGL;

class Engine::Scene {
protected:
    std::string name;
    Hud* heads_up_display = nullptr;
    Camera* active_camera = nullptr;

    std::vector<InstancedTexture*> instanced_textures{};
    std::vector<InstancedTexture*> instanced_alpha_masks{};

    std::vector<RenderTarget*> layers{};
    std::vector<Fixed*> fixed_list{};
    std::vector<Entity*> entity_list{};
protected:
    [[nodiscard]] std::vector<Entity*> GetFlatEntityList(const std::vector<Entity*>& entity_list) const;
public:
    [[nodiscard]] bool EntityListContains(const Entity* entity) const;
    [[nodiscard]] bool FixedListContains(const Fixed* fixed) const;
    [[nodiscard]] size_t FixedCount() const;
    [[nodiscard]] size_t EntityCount() const;
    [[nodiscard]] std::string GetName() const;
    [[nodiscard]] Camera* GetActiveCamera() const;
    [[nodiscard]] Texture* GetInstancedTexture(const InstancedSprite* user);
    [[nodiscard]] Texture* GetInstancedAlphaMask(const InstancedSprite* user);
public:
    void AppendEntity(Entity* entity);
    void AppendFixed(Fixed* fixed);

    // Removes and deallocates.
    void DestroyEntity(Entity* entity);
    void DestroyFixed(Fixed* fixed);

    // Only removes from the list.
    void RemoveEntity(Entity* entity);
    void RemoveFixed(Fixed* fixed);

    virtual void Unload();
    virtual void Init() {}
    virtual void Update();
    virtual void Render(RenderTarget* render_target = nullptr);
public:
    explicit Scene(const std::string& name) : name(name) {}
    virtual ~Scene();
};
